﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;
using System.IO;
using FluxJpeg.Core;
using System.Windows.Browser;
using System.Threading;


namespace OpacityMaskTester
{
    public partial class MainPage : UserControl
    {

        RGB CurrentFill = new RGB("255;0;0");
        int NumImagesLoaded = 0;
        const int TotalImageNumber = 2;

        public MainPage()
        {
            InitializeComponent();
            InitWheels();
            BackgroundImage.ImageOpened += new EventHandler<RoutedEventArgs>(BackgroundImage_ImageOpened);
            AideFillMaskImage.ImageOpened += new EventHandler<RoutedEventArgs>(AideFillMaskImage_ImageOpened);
        }

        void AideFillMaskImage_ImageOpened(object sender, RoutedEventArgs e)
        {
            LoadingText.Text += "\nImage : Opacity Mask Image : LOADED";
            NumImagesLoaded++;
            AideFillMaskImage.UpdateLayout();
            Dispatcher.BeginInvoke(UpdateLoaded);
        }

        void BackgroundImage_ImageOpened(object sender, RoutedEventArgs e)
        {
            LoadingText.Text += "\nImage : Background Image : LOADED";
            NumImagesLoaded++;
            BackgroundImage.UpdateLayout();
            Dispatcher.BeginInvoke(UpdateLoaded);
        }

        private void UpdateLoaded()
        {
            if (NumImagesLoaded == TotalImageNumber)
            {
                if (BackgroundImage.ActualWidth==0) {
                    BackgroundImage.Width=500;
                    BackgroundImage.Height = 375;
                }
                FillRectangle.Width = BackgroundImage.ActualWidth;
                FillRectangle.Height = BackgroundImage.ActualHeight;
                Result.Width = BackgroundImage.ActualWidth;
                AideResult.Width = BackgroundImage.ActualWidth / 2;
                AideResult.Height = BackgroundImage.ActualHeight / 2;
                AideResult2.Height = BackgroundImage.ActualHeight / 2;
                AideResult3.Height = BackgroundImage.ActualHeight / 2;
                AideResult2.Width = BackgroundImage.ActualWidth / 2;
                AideResult3.Width = BackgroundImage.ActualWidth / 2;
                AideFillRectangle.Width = BackgroundImage.ActualWidth;
                AideFillRectangle.Height = BackgroundImage.ActualHeight;
                ContentSP.Visibility = Visibility.Visible;
                LoadingText.Visibility = Visibility.Collapsed;
            }
        }

        private void InitWheels()
        {
            JogWheel JWR = new JogWheel(new SolidColorBrush(Colors.Red));
            JWR.Restrict = true;
            JWR.SetValue(Canvas.NameProperty, "RedWheel");
            JWR.Update_Event+=new JogWheel.Update_EventHandler(RGBWheel_Update_Event);
            JWR.ResetAngle(-1);
            RGBwheels.Children.Add(JWR);
            JogWheel JWG = new JogWheel(new SolidColorBrush(Colors.Green));
            JWG.Restrict = true;
            JWG.SetValue(Canvas.NameProperty, "GreenWheel");
            JWG.Update_Event += new JogWheel.Update_EventHandler(RGBWheel_Update_Event);
            RGBwheels.Children.Add(JWG);
            JogWheel JWB = new JogWheel(new SolidColorBrush(Colors.Blue));
            JWB.Restrict = true;
            JWB.SetValue(Canvas.NameProperty, "BlueWheel");
            JWB.Update_Event += new JogWheel.Update_EventHandler(RGBWheel_Update_Event);
            RGBwheels.Children.Add(JWB);
        }


        void RGBWheel_Update_Event(object sender, MouseEventArgs e)
        {
            JogWheel jw = (JogWheel)sender;
            switch (jw.Name)
            {
                case "RedWheel" :
                    UpdateFill(jw.Range, -1, -1);
                    break;
                case "GreenWheel": UpdateFill(-1,jw.Range, -1);
                    break;
                case "BlueWheel": UpdateFill(-1, -1,jw.Range );
                    break;
                default:
                    break;
            }

        }

        private void UpdateFill(double R, double G, double B)
        {
            Color NewFillColor = new Color();
            NewFillColor.A = 255;
            NewFillColor.R = (byte)CurrentFill.Red;
            NewFillColor.G = (byte)CurrentFill.Green;
            NewFillColor.B = (byte)CurrentFill.Blue;
            if (R > 0) CurrentFill.Red = ConvertDoubleToByte(R);
            if (G > 0) CurrentFill.Green = ConvertDoubleToByte(G);
            if (B > 0) CurrentFill.Blue = ConvertDoubleToByte(B);
            FillRectangle.Fill = new SolidColorBrush(NewFillColor);
            AideFillRectangle.Fill = new SolidColorBrush(NewFillColor);
        }

        /// <summary>
        /// Numeric domain transfer
        /// </summary>
        /// <param name="R">0.0 - 1.0</param>
        /// <returns>0 - 255</returns>
        private byte? ConvertDoubleToByte(double range)
        {
            double raw = range / (1.0 / (255 - 1));
            int index = (int)Math.Round(raw);
            return (byte)index;
        }

        private void LoadBackgroundButton_Click(object sender, RoutedEventArgs e)
        {
            BitmapImage BM = LoadUtils.LoadNewImage();
            BackgroundImage.Source = BM;
            AideBackgroundImage.Source = BM;
            Dispatcher.BeginInvoke(UpdateLoaded);
        }

        private void LoadFillImageButton_Click(object sender, RoutedEventArgs e)
        {
            BitmapImage BM = LoadUtils.LoadNewImage();
            FillImage.Source = BM;
            AideFillImage.Source = BM;
            FillImage.Visibility = Visibility.Visible;
            FillRectangle.Visibility = Visibility.Collapsed;
            AideFillImage.Visibility = Visibility.Visible;
            AideFillRectangle.Visibility = Visibility.Collapsed;
            RemoveFillImage.Visibility = Visibility.Visible;
        }

        private void LoadOpacityMaskButton_Click(object sender, RoutedEventArgs e)
        {
            BitmapImage BM = LoadUtils.LoadNewImage();
            FillMaskImage.ImageSource = BM;
            AideFillMaskImage.Source = BM;
        }

        private void RemoveFillImage_Click(object sender, RoutedEventArgs e)
        {
            FillImage.Visibility = Visibility.Collapsed;
            FillRectangle.Visibility = Visibility.Visible;
            AideFillImage.Visibility = Visibility.Collapsed;
            AideFillRectangle.Visibility = Visibility.Visible;
            RemoveFillImage.Visibility = Visibility.Collapsed;
        }

        private void SaveButton_Click(object sender, RoutedEventArgs e)
        {
            Rectangle r = new Rectangle();
            r.Fill = new SolidColorBrush(Colors.White);
            r.Width = BackgroundImage.ActualWidth;
            r.Height = BackgroundImage.ActualHeight;
            TextBlock tb = new TextBlock();
            tb.Text = "Image created from " + HtmlPage.Document.DocumentUri.ToString();
            tb.Margin = new Thickness(0, 0, 0, 12.0);
            tb.HorizontalAlignment = HorizontalAlignment.Center;
            tb.VerticalAlignment = VerticalAlignment.Bottom;
            tb.Foreground = new SolidColorBrush(Colors.Blue);
            Grid g = new Grid();
            g.Width = r.Width;
            g.Height = r.Height;
            g.Children.Add(r);
            Result.Children.Remove(Savable);
            g.Children.Add(Savable);
            g.Children.Add(tb);
            

            WriteableBitmap bitmap = new WriteableBitmap(g, null);
            g.Children.Remove(Savable);
            Result.Children.Add(Savable);
            System.Windows.Controls.Image image = new System.Windows.Controls.Image();
            image.Width = 100;
            image.Height = 100;
            image.Source = bitmap;
            if (bitmap != null)
            {
                SaveFileDialog saveDlg = new SaveFileDialog();
                saveDlg.Filter = "JPEG Files (*.jpeg)|*.jpeg";
                saveDlg.DefaultExt = ".jpeg";

                if ((bool)saveDlg.ShowDialog())
                {
                    using (Stream fs = saveDlg.OpenFile())
                    {

                        SaveToFile(bitmap, fs);
                    }
                }
            }
            g = null;
        }

        private static void SaveToFile(WriteableBitmap bitmap, Stream fs)
        {
            int width = bitmap.PixelWidth;
            int height = bitmap.PixelHeight;
            int bands = 3;
            byte[][,] raster = new byte[bands][,];

            //Convert the Image to pass into FJCore
            //Code From http://stackoverflow.com/questions/1139200/using-fjcore-to-encode-silverlight-writeablebitmap
            for (int i = 0; i < bands; i++)
            {
                raster[i] = new byte[width, height];
            }

            for (int row = 0; row < height; row++)
            {
                for (int column = 0; column < width; column++)
                {
                    int pixel = bitmap.Pixels[width * row + column];
                    raster[0][column, row] = (byte)(pixel >> 16);
                    raster[1][column, row] = (byte)(pixel >> 8);
                    raster[2][column, row] = (byte)pixel;
                }
            }

            ColorModel model = new ColorModel { colorspace = ColorSpace.RGB };
            FluxJpeg.Core.Image img = new FluxJpeg.Core.Image(model, raster);

            //Encode the Image as a JPEG
            MemoryStream stream = new MemoryStream();
            FluxJpeg.Core.Encoder.JpegEncoder encoder = new FluxJpeg.Core.Encoder.JpegEncoder(img, 100, stream);
            encoder.Encode();

            //Back to the start
            stream.Seek(0, SeekOrigin.Begin);

            //Get teh Bytes and write them to the stream
            byte[] binaryData = new Byte[stream.Length];
            long bytesRead = stream.Read(binaryData, 0, (int)stream.Length);
            fs.Write(binaryData, 0, binaryData.Length);
        }
    }
}
